/*-- ***** BEGIN LICENSE BLOCK *****
 -   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 -
 - The contents of this file are subject to the Mozilla Public License Version
 - 1.1 (the "License"); you may not use this file except in compliance with
 - the License. You may obtain a copy of the License at
 - http://www.mozilla.org/MPL/
 -
 - Software distributed under the License is distributed on an "AS IS" basis,
 - WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 - for the specific language governing rights and limitations under the
 - License.
 -
 - The Original Code is Environment Proxy.
 -
 - The Initial Developer of the Original Code is
 - Peter Leugner.
 - Portions created by the Initial Developer are Copyright (C) 2006
 - the Initial Developer. All Rights Reserved.
 -
 - Contributor(s):
 -
 - Alternatively, the contents of this file may be used under the terms of
 - either the GNU General Public License Version 2 or later (the "GPL"), or
 - the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 - in which case the provisions of the GPL or the LGPL are applicable instead
 - of those above. If you wish to allow use of your version of this file only
 - under the terms of either the GPL or the LGPL, and not to allow others to
 - use your version of this file under the terms of the MPL, indicate your
 - decision by deleting the provisions above and replace them with the notice
 - and other provisions required by the GPL or the LGPL. If you do not delete
 - the provisions above, a recipient may use your version of this file under
 - the terms of any one of the MPL, the GPL or the LGPL.
 -
 - ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

// Gecko 1.9.0/1.9.1 compatibility - add XPCOMUtils.defineLazyServiceGetter
if (!("defineLazyServiceGetter" in XPCOMUtils)) {
	XPCOMUtils.defineLazyServiceGetter = function XPCU_defineLazyServiceGetter(obj, prop, contract, iface) {
		obj.__defineGetter__(prop, function XPCU_serviceGetter() {
			delete obj[prop];
			return obj[prop] = Cc[contract].getService(Ci[iface]);
		});
	};
}

XPCOMUtils.defineLazyServiceGetter(this, "ENV", "@mozilla.org/process/environment;1", "nsIEnvironment");
XPCOMUtils.defineLazyServiceGetter(this, "PREFS", "@mozilla.org/preferences-service;1", "nsIPrefBranch2");

const source = "extensions.envproxy.";
const target = "network.proxy.";

function EnvProxyService() {
	this.type = 0;
	this.setProxyForProtocol('http');
	this.setProxyForProtocol('ssl');
	this.setProxyForProtocol('ftp');
	this.setProxyForProtocol('socks');
	this.setProxyExceptions();
	this.setSocksVersion();
	this.setProxyAutoConfig();
	
	PREFS.setIntPref('network.proxy.type', this.type);
	PREFS.addObserver('extensions.envproxy', this, false);
}

EnvProxyService.prototype = {

	observe: function(subject, topic, data) {
		if (topic == "nsPref:changed") {
			var protocol = data.split('.')[2];
			switch (protocol) {
				case 'noproxy':
					this.setProxyExceptions();
					break;
				case 'socks_version':
					this.setSocksVersion();
					break;
				case 'autoconfig_url':
					this.setAutoConfig();
					break;
				default:
					this.setProxyForProtocol(protocol);
					break;
			}
			PREFS.setIntPref('network.proxy.type', this.type);
		}
	},
	
	setProxyForProtocol: function(aProtocol) {
		try {
			var variable = PREFS.getCharPref(source + aProtocol);
			if (ENV.exists(variable)) {
				url = this.parseURL(ENV.get(variable));
				PREFS.setCharPref(target + aProtocol, url.host);
				PREFS.setIntPref(target + aProtocol + '_port', url.port);
				this.type = 1;
			}
		} catch (ex) {
			dump(ex);
		}
	},
	
	setProxyExceptions: function() {
		try {
			var variable = PREFS.getCharPref(source + 'noproxy');
			if (ENV.exists(variable)) {
				PREFS.setCharPref('network.proxy.no_proxies_on', ENV.get(variable));
			}
		} catch (ex) {
			dump(ex);
		}
	},
	
	setSocksVersion: function() {
		try {
			var variable = PREFS.getCharPref(source + 'socks_version');
			if (ENV.exists(variable)) {
				PREFS.setIntPref('network.proxy.socks_version', ENV.get(variable));
			}
		} catch (ex) {
			dump(ex);
		}
	},
	
	setProxyAutoConfig: function() {
		try {
			var variable = PREFS.getCharPref(source + 'autoconfig');
			if (ENV.exists(variable)) {
				PREFS.setCharPref('network.proxy.autoconfig_url', ENV.get(variable));
				this.type = 2;
			}
		} catch (ex) {
			dump(ex);
		}
	},
		
	parseURL: function(url) {
		var e=/^((.*):\/\/)?([^:/\s]+)(:(\d*))?/;
		return url.match(e) ? {
			protocol: RegExp.$2,
			host: RegExp.$3,
			port: RegExp.$5
		} : {
			protocol: "",
			host: "",
			port: 0
		};
	},
	
	classDescription: "@as-computer.de/env-proxy-service;1",
	classID: Components.ID("{65d74710-7429-11db-9fe1-0800200c9a66}"),
	contractID: "@as-computer.de/env-proxy-service;1",
	QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver, Ci.nsISupportsWeakReference]),
	_xpcom_categories: [{
		category: "profile-after-change"
	}],
	_xpcom_factory: {
		createInstance: function() {
			if (EnvProxyService.instance == null) {
				EnvProxyService.instance = new EnvProxyService();
			}
			return EnvProxyService.instance;
		}
	}

}

if (XPCOMUtils.generateNSGetFactory) {
	var NSGetFactory = XPCOMUtils.generateNSGetFactory([EnvProxyService]);
} else {
	var NSGetModule = XPCOMUtils.generateNSGetModule([EnvProxyService]);
}